
#include <windows.h>
#include "editor.h"

bool Editor::InitializeSurface(HWND hFrame, unsigned width, int height)
{
	_parentHandle = hFrame;
	_width = width;
	_height = height;
	HDC framedc = GetDC(hFrame);
	_surfacedc = CreateCompatibleDC(framedc);
	if (_surfacedc == NULL) {
		ReleaseDC(hFrame, framedc);
		return false;
	}
	_screen_width = 1280;
	_screen_height = 1024;
	_surfacebm = CreateCompatibleBitmap(framedc, _screen_width, _screen_height);
	ReleaseDC(hFrame, framedc);
	if (_surfacebm == NULL) {
		DeleteDC(_surfacedc);
		return false;
	}
	SelectObject(_surfacedc, (HGDIOBJ) _surfacebm);

	_selMarqueeShow = false;
	RECT surfaceRect = { 0, 0, height, width };
	SetBkColor(_surfacedc, RGB(0, 0, 0));
	ExtTextOut(_surfacedc, 0, 0, ETO_OPAQUE, &surfaceRect, 0, 0, 0);
	InvalidateRect(hFrame, NULL, FALSE);
	return true;
}

void Editor::Release()
{
	DeleteDC(_surfacedc);
	DeleteObject((HGDIOBJ) _surfacebm);
}

bool Editor::Paint(HWND hFrame, HDC hdc, RECT position)
{
	_position = position;
	/* Avoid flickerering by excluding area */
	if (_selMarqueeShow == true) {
		ExcludeMarquee(hdc);
	}
	BitBlt(hdc, 0, 0, _width, _height, _surfacedc, 0, 0, SRCCOPY);
	SelectClipRgn(hdc, NULL);
	if (_selMarqueeShow == true) {
		DrawMarquee(hdc);
	}
	return true;
}

void Editor::ClickDown(unsigned buttons, POINTS position)
{
	if (!MonoSelection(position.x, position.y)) {
		if (buttons & _selectionMarqueeButtons) {
			SetCapture(_parentHandle);
			_selMarqueeActivate = true;
			_selMarqueeShow = true;
			position.x += _position.left;
			position.y += _position.top;
			_selMarqueeStartPosition = position;
		}
	}
}

void Editor::ClickUp(unsigned buttons, POINTS position)
{
	
}

void Editor::MouseMove(unsigned buttons, POINTS position)
{
	if (_selMarqueeActivate == true) {
		if (buttons & _selectionMarqueeButtons) {
			RECT beforeMarqueeRect = GetMarqueeRect();
			_selMarqueeEndPosition = position;
			_selMarqueeEndPosition.x += _position.left;
			_selMarqueeEndPosition.y += _position.top;
			RECT afterMarqueeRect = GetMarqueeRect();
			if (EqualRect(&beforeMarqueeRect, &afterMarqueeRect)) {
				InvalidateRect(_parentHandle, NULL, FALSE);
			}
		}
		else {
			ReleaseCapture();
			_selMarqueeActivate = false;
		}
	}
}

RECT Editor::GetMarqueeRect()
{
	RECT rect = { -_position.left, -_position.top, -_position.left, -_position.top }; //left, top, right, bottom
	rect.bottom += _selMarqueeStartPosition.y / _selectionMarqueeSize * _selectionMarqueeSize;
	rect.left += _selMarqueeStartPosition.x / _selectionMarqueeSize * _selectionMarqueeSize;
	rect.top += (_selMarqueeEndPosition.y + _selectionMarqueeSize) / _selectionMarqueeSize * _selectionMarqueeSize;
	rect.right += (_selMarqueeEndPosition.x + _selectionMarqueeSize) / _selectionMarqueeSize * _selectionMarqueeSize;
	return rect;
}

void Editor::ExcludeMarquee(HDC hdc)
{
	RECT marquee = GetMarqueeRect();
	//top line	
	ExcludeClipRect(hdc, marquee.left, marquee.top, marquee.right, marquee.top - 1);
	//left line
	ExcludeClipRect(hdc, marquee.left, marquee.top, marquee.left + 1, marquee.bottom);
	//right line
	ExcludeClipRect(hdc, marquee.right - 1, marquee.top, marquee.right, marquee.bottom);
	//bottom line
	ExcludeClipRect(hdc, marquee.left, marquee.bottom + 1, marquee.right, marquee.bottom);
}

void Editor::DrawMarquee(HDC hdc)
{
	HPEN marqueePen = CreatePen(PS_DOT, 1, RGB(0, 0, 0));
	HPEN oldPen = (HPEN) SelectObject(hdc, (HGDIOBJ) marqueePen);
	HBRUSH oldBrush = (HBRUSH) SelectObject(hdc, (HGDIOBJ) GetStockObject(BLACK_BRUSH));
	RECT marqueeRect = GetMarqueeRect();
	//Rectangle(hdc, marqueeRect.left
	MoveToEx(hdc, marqueeRect.left, marqueeRect.bottom, NULL);
	LineTo(hdc, marqueeRect.left, marqueeRect.top - 1);
	LineTo(hdc, marqueeRect.right - 1, marqueeRect.top - 1);
	LineTo(hdc, marqueeRect.right - 1, marqueeRect.bottom);
	LineTo(hdc, marqueeRect.left, marqueeRect.bottom);
	SelectObject(hdc, (HGDIOBJ) oldPen);
	SelectObject(hdc, (HGDIOBJ) oldBrush);
	DeleteObject((HGDIOBJ) marqueePen);
}

bool Editor::Render(RECT position)
{
	// TODO: Use brush instead

//	HBITMAP uncharted = CreateBitmap(26, 26
	return true;
}
