
#ifndef _EDITOR_H_
#define _EDITOR_H_

class Editor
{
public:
	Editor() { }
	~Editor() { }

	/*
	 * Initialize the editor's surface
	 */
	virtual bool InitializeSurface(HWND hFrame) = 0;
	bool InitializeSurface(HWND hFrame, unsigned width, int height);
	/*
	 * Used to repaint the surface onto the frame
	 */
	bool Paint(HWND hFrame, HDC hdc, RECT position);
	virtual bool Render(RECT position);
	/*
	 * Called when the zone is mouse-clicked.
	 */
	void ClickDown(unsigned buttons, POINTS position);
	void ClickUp(unsigned buttons, POINTS position);
	void MouseMove(unsigned buttons, POINTS position);
	/*
	 * MonoSelection :	Called when a selection click occure
	 * @return			true if an element has been selected, otherwise the marquee will be enable.
	 * MultiSelection :	Called each time that marquee is resized.
	 * @return			true if one or more elements has been selected.
	 */
	virtual bool MonoSelection(int x, int y) = 0;
	virtual bool MultiSelection(RECT position) = 0;
	/*
	 * Return the resource id of the menu to be used. 0 if none
	 */
	virtual unsigned GetMenuId() const = 0;
	virtual const char *GetMenuName() const = 0;
	/*
	 * Free allocated memory
	 */
	virtual void Release();

	unsigned	GetWidth() const { return _width; }
	unsigned	GetHeight() const { return _height; }

protected:
	// Handle, must be deleted using DeleteObject
	HBITMAP		_surfacebm;
	HDC			_surfacedc;
	HWND		_parentHandle;
	// Dimensions of the surface
	unsigned	_width;
	unsigned	_height;
	unsigned	_screen_width;
	unsigned	_screen_height;
	/*unsigned	_pos_x; // Current x scroll position (Updated on a Paint call)
	unsigned	_pos_y; // Current y scroll position*/
	RECT		_position;
	// Must be initialized
	unsigned	_selectionMarqueeButtons; // Buttons for which the marquee is used.
	unsigned	_selectionMarqueeSize; // Size in pixel of a marquee unit

private:
	RECT		GetMarqueeRect(); // Get a RECT from the marquee POINTS start and end.
	void		ExcludeMarquee(HDC hdc); // Exclude the marquee to avoid flicker
	void		DrawMarquee(HDC hdc); // Draw the marquee

	bool		_selMarqueeActivate; //Is the selection marquee activated?
	bool		_selMarqueeShow;
	POINTS		_selMarqueeStartPosition; // Initiale position of the marquee
	POINTS		_selMarqueeEndPosition;
};

#endif /* _EDITOR_H_ */
