
#include <windows.h>
#include <commctrl.h>
#include "frame.h"
#include "overworld.h"
#include "level.h"
#include "rom.h"
#include "resource.h"
#include "gfxviewer.h"

const char *Frame::WndFrameClassName	= "DKC2Frame";
const char *Frame::WndFrameTitle		= "DKC2 Editor";

bool Frame::Initialize(HINSTANCE hInstance)
{
	WNDCLASS wc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hbrBackground = GetSysColorBrush(COLOR_APPWORKSPACE);
	wc.hCursor = LoadCursor(hInstance, IDC_ARROW);
	wc.hIcon = LoadIcon(hInstance, IDI_APPLICATION);
	wc.hInstance = hInstance;
	wc.lpfnWndProc = Frame::WndCallbackProc;
	wc.lpszClassName = WndFrameClassName;
	wc.lpszMenuName = MAKEINTRESOURCE(IDR_FRMMENU);
	wc.style = NULL;
	RegisterClass(&wc);
	_hwnd = CreateWindow(WndFrameClassName, WndFrameTitle, WS_VISIBLE | WS_OVERLAPPEDWINDOW | WS_VSCROLL | WS_HSCROLL | WS_CLIPSIBLINGS,
		200, 200, 300, 300, NULL, NULL, hInstance, 0);
	Window::Initialize();
	// Status Bar cannot be supported because the main window is using scrollbars
	//_hStatusBar = CreateWindowEx(0, STATUSCLASSNAME, NULL, SBARS_SIZEGRIP | WS_CHILD | WS_VISIBLE, 0, 0, 0, 0,
	//	_hwnd, (HMENU) 1111, hInstance, 0);
	_animation = true;
	SetTimer(_hwnd, 10, 100, NULL);
	return _hwnd != NULL;
}

LRESULT Frame::WndProc(UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
	case WM_PAINT:
		if (_editor != 0) {
			RECT rectSize;
			GetClientRect(_hwnd, &rectSize);
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint(_hwnd, &ps);
			RECT editorRect = GetEditorRect();
			ExcludeClipRect(hdc, editorRect.left, editorRect.top, editorRect.right, editorRect.bottom);
			ExtTextOut(hdc, 0, 0, ETO_OPAQUE, &rectSize, 0, 0, 0);
			SelectClipRgn(hdc, NULL);
			_editor->Render(GetEditorPosition());
			_editor->Paint(_hwnd, hdc, GetEditorPosition());
			EndPaint(_hwnd, &ps);
			return 0;
		}
		break ;
	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case ID_FILE_OPEN:
			if (!rom.OpenFile(_hwnd)) {
				return 0;
			}
			EditorLevel();
			return 0;
		case ID_FILE_CLOSE:
			CloseEditor();
			rom.CloseFile();
			return 0;
		case ID_FILE_GFXVIEWER:
			OpenGraphicViewer(_hwnd);
			return 0;
		}
		break ;
	case WM_LBUTTONDOWN:
		if (_editor != 0) {
			_editor->ClickDown(wParam, MAKEPOINTS(lParam));
		}
	case WM_MOUSEMOVE:
		if (_editor != 0) {
			_editor->MouseMove(wParam, MAKEPOINTS(lParam));
		}
		return 0;
	case WM_ERASEBKGND:{
			RECT size;
			GetClientRect(_hwnd, &size);
			
			return 0;
		}
	case WM_SIZE:
		UpdateScroll(0, 0);
		UpdateScroll(0, 0);
		return 0;
	case WM_VSCROLL:
		if (_editor != 0) {
			int nScrollCode = LOWORD(wParam);
			switch (nScrollCode) {
			case SB_THUMBTRACK:
				UpdateScroll(0, HIWORD(wParam) - _editor_y);
				break ;
			case SB_LINEDOWN:
				UpdateScroll(0, 5);
				break ;
			case SB_LINEUP:
				UpdateScroll(0, -5);
				break ;
			}
			return 0;
		}
	case WM_HSCROLL:
		if (_editor != 0) {
			int nScrollCode = LOWORD(wParam);
			switch (nScrollCode) {
			case SB_THUMBTRACK:
				UpdateScroll(HIWORD(wParam) - _editor_x, 0);
				break ;
			case SB_LINERIGHT:
				UpdateScroll(5, 0);
				break ;
			case SB_LINELEFT:
				UpdateScroll(-5, 0);
				break ;
			}
			return 0;
		}
	case WM_TIMER:
		if (_editor != 0) {
			_editor->Render(GetEditorPosition());
			HDC hdc = GetDC(_hwnd);
			_editor->Paint(_hwnd, hdc, GetEditorPosition());
			ReleaseDC(_hwnd, hdc);
		}
		break ;
	case WM_CLOSE:
		PostQuitMessage(1);
		return 0;
	}
	return DefWindowProc(_hwnd, msg, wParam, lParam);
}

void Frame::CloseEditor()
{
	if (_editor != 0) {
		_editor->Release();
		delete _editor;
		_editor = 0;
		UpdateScroll(0, 0);
		if (_edMenu != 0) {
			DeleteMenu(GetMenu(_hwnd), 1, MF_BYPOSITION);
			DestroyMenu(_edMenu);
			_edMenu = 0;
			DrawMenuBar(_hwnd);
		}
	}
}

bool Frame::EditorOverworld()
{
	_editor = new Overworld();
	LoadEditorMenu();
	if (!_editor->InitializeSurface(_hwnd)) {
		CloseEditor();
		return false;
	}
	UpdateScroll(0, 0);
	return true;
}

bool Frame::EditorLevel()
{
	_editor = new Level();
	LoadEditorMenu();
	if (!_editor->InitializeSurface(_hwnd)) {
		CloseEditor();
		return false;
	}
	UpdateScroll(0, 0);
	return true;
}

void Frame::LoadEditorMenu()
{
	unsigned resid = _editor->GetMenuId();
	if (resid == 0) {
		return ;
	}
	HMENU edMenu = LoadMenu((HINSTANCE) GetModuleHandle(NULL), MAKEINTRESOURCE(resid));
	HMENU toolMenu = GetMenu(_hwnd);
	
	MENUITEMINFO mi = { 0 };
	mi.cbSize = sizeof(mi);
	mi.hSubMenu = edMenu;
	mi.fMask = MIIM_TYPE | MIIM_ID | MIIM_SUBMENU;
	mi.fType = MFT_STRING;
	mi.wID = 260;
	mi.dwTypeData = const_cast<char *>(_editor->GetMenuName());
	InsertMenuItem(toolMenu, 1, TRUE, &mi);
	_edMenu = GetSubMenu(toolMenu, 1);
	DrawMenuBar(_hwnd);
}

void Frame::UpdateScroll(int x, int y)
{
	SCROLLINFO si;
	if (_editor == 0) {
		si.fMask = SIF_RANGE;
		si.nPos = 0;
		SetScrollInfo(_hwnd, SB_VERT, &si, TRUE); 
		SetScrollInfo(_hwnd, SB_HORZ, &si, TRUE); 
		return ;
	}
	// Get the window dimensions
	RECT rectSize;
	GetClientRect(_hwnd, &rectSize);
	si.cbSize = sizeof(si);
	// The vertical scroll bar
	unsigned heightMax = _editor->GetHeight();
	si.fMask = SIF_POS;
	GetScrollInfo(_hwnd, SB_VERT, &si);
	si.fMask = 0;
	unsigned bottomPos = _editor_y + y + rectSize.bottom;
	y = _editor_y + y < 0 ? - _editor_y : y;
	y = bottomPos > heightMax ? 0 : y;
	if (_editor_y != si.nPos) {
		y = si.nPos - _editor_y;
	}
	// Vertically scroll the window?
	if (y) {
		_editor_y += y;
		si.fMask |= SIF_POS;
		si.nPos = _editor_y;
	}
	si.fMask |= SIF_RANGE | SIF_PAGE;
	si.nMax = heightMax;
	si.nMin = 0;
	si.nPage = rectSize.bottom - 1;
	SetScrollInfo(_hwnd, SB_VERT, &si, TRUE); 
	// The horizontal scroll bar
	unsigned widthMax = _editor->GetWidth();
	si.fMask = SIF_POS;
	GetScrollInfo(_hwnd, SB_HORZ, &si);
	si.fMask = 0;
	unsigned rightPos = _editor_x + x + rectSize.right;
	x = _editor_x + x < 0 ? 0 : x;
	x = rightPos > widthMax ? 0 : x;
	if (_editor_x != si.nPos) {
		x = si.nPos - _editor_x;
	}
	// Horizontally scroll the window
	if (x) {
		_editor_x += x;
		si.fMask |= SIF_POS;
		si.nPos = _editor_x;
	}
	si.fMask |= SIF_RANGE | SIF_PAGE;
	si.nMax = widthMax;
	si.nMin = 0;
	si.nPage = rectSize.right - 1;
	if (x || y) {
		ScrollWindow(_hwnd, -x, -y, NULL, NULL);
	}
	SetScrollInfo(_hwnd, SB_HORZ, &si, TRUE);
}

RECT Frame::GetEditorRect() const
{
	RECT edzone;
	edzone.left = 0;
	edzone.top = 0;
	edzone.bottom = _editor->GetHeight();
	edzone.right = _editor->GetWidth();
	edzone.right -= _editor_x;
	edzone.bottom -= _editor_y;
	return edzone;
}

RECT Frame::GetEditorPosition()
{
	RECT dimension;
	GetClientRect(_hwnd, &dimension);
	RECT position = { _editor_x, _editor_y, _editor_x + dimension.right, _editor_x + dimension.top };
	return position;
}
