
#include <stdio.h>
#include <windows.h>
#include <crtdbg.h>

#include "level.h"
#include "rom.h"
#include "sprites.h"

bool Level::InitializeSurface(HWND hFrame)
{
	_selectionMarqueeButtons = WM_LBUTTONDOWN;
	_selectionMarqueeSize = 16;
	Editor::InitializeSurface(hFrame, 20000, 20000);
	LoadLevel(0x04);
	return true;
}

bool Level::MonoSelection(int x, int y)
{
	return false;
}

bool Level::MultiSelection(RECT position)
{
	return true;
}

bool Level::LoadLevel(unsigned id)
{
	_level_id = id;
	ROMFILE rf = rom.GetROMFILE();
	rf.FollowPtr2(0x3D0000, id * 2);
	if (rf.cAddr == 0x3D0000)
		return false;
	rf.Read(&_level_header, sizeof(_level_header));
	rf.Read(0x3D0000 | _level_header.levelTemplate, &_level_template, sizeof(_level_template));
	rf.FollowPtr2(0x3D819A, _level_template.gfx * 2);
	LoadSprites();
#ifdef _DEBUG
	unsigned palette_addr = 0x3D1610;
	rf.Read(0x35BC2A + id * 2, &palette_addr, 2);
	_level_regs.Load(0x06);
	_level_vram.LoadPalette(palette_addr);
	_level_vram.LoadPack(0x01);
	_level_vram.LoadPack(0x07);
#endif // _DEBUG
	InvalidateRect(_parentHandle, NULL, FALSE);
	return true;
}

bool Level::LoadSprites()
{
	ROMFILE rf = rom.GetROMFILE();
	// How many sprites? Substract the next pointer to figure out.
	unsigned diff1, diff2;
	diff1 = diff2 = 0;
	rf.Read(0x3E0000 + _level_id * 2, &diff1, 2);
	rf.Read(0x3E0000 + _level_id * 2 + 2, &diff2, 2);
	unsigned cbLvlSprites = diff2 - diff1;
	_level_sprites_total = cbLvlSprites / 8;
	rf.FollowPtr2(0x3E0000, _level_id * 2);
	rf.Read(&_level_sprites, cbLvlSprites);
	for (int i = 0; i < _level_sprites_total; i++) {
		_level_sprites_gfx[i].Load(_level_sprites.sprite[i].id,
			_level_sprites.sprite[i].x, _level_sprites.sprite[i].y);
	}
	return true;
}

bool Level::DrawSprites(HDC hdc, RECT position, int base_x, int base_y)
{
	for (int i = 0; i < _level_sprites_total; i++) {
		unsigned sprite_x = _level_sprites.sprite[i].x + base_x - position.left,
			sprite_y = _level_sprites.sprite[i].y + base_y - position.top;
		_level_sprites_gfx[i].Draw(hdc, sprite_x, sprite_y);
	}
	return true;
}

bool Level::Render(RECT position)
{
	RECT rect = { 0, 0, _screen_width, _screen_height };
	SetBkColor(_surfacedc, RGB(0, 0, 0));
	ExtTextOut(_surfacedc, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
	DrawSprites(_surfacedc, _position, 0, 0);
	Editor::Render(position);
	//_level_vram.DrawBG2(_surfacedc, 0, 0, _width, 224);
//	test.RenderNextFrame(_surfacedc, 20, 20);
	/*
	for (int i = 0; i < _level_sprites_total; i++)
	{
		if (!DrawSprite(_surfacedc, _level_sprites.sprite[i].sp,
			_level_sprites.sprite[i].x - 400, _level_sprites.sprite[i].y - 350))
		{
			char t[17];
			sprintf(t, "%.4X \\ 2 = %.4X\r\n", _level_sprites.sprite[i].sp, _level_sprites.sprite[i].sp / 2);
			DWORD written;
			WriteFile(tmp, t, 17, &written, NULL);
		}
	}*/
	//
	return true;
}
