
#ifndef _LEVEL_H_
#define _LEVEL_H_

#include "editor.h"
#include "resource.h"
#include "rom.h"
#include "registers.h"
#include "vram.h"
#include "sprites.h"

#pragma pack(push, 1)

struct LEVELHEADER
{
	unsigned short	unknown00;
	LOCALADDR		levelTemplate;
	unsigned short	layer2id;
	unsigned short	unknown06;
	unsigned short	unknown08;
	unsigned short	unknown0A;
	unsigned short	startPosX;
	unsigned short	startPosY;
};

struct LEVELTEMPLATE
{
	unsigned short	unknown00;	// Level type related?
	unsigned short	unknown02;	// Level type related?
	unsigned short	music;		// Music id
	unsigned short	palette;		// Palette related (id?)
	unsigned short	script;		// Seem to be used with a jmp
	unsigned short	unknown0A;
	unsigned char	reg;		// Register id for the register pack.
	unsigned char	gfx;		// GFX ID for the gfx pack
	unsigned short	unknown0E;
	unsigned short	unknown10;
};

struct SPRITES
{
	struct
	{
		unsigned short	header;
		unsigned short	x;
		unsigned short	y;
		LOCALADDR		id;
	} sprite[500];
};


#pragma pack(pop)

class Level : public Editor
{
public:
	Level() : _level_vram(&_level_regs) { }
	~Level() { }

	bool InitializeSurface(HWND hFrame);
	bool MonoSelection(int x, int y);
	bool MultiSelection(RECT position);
	unsigned GetMenuId() const { return IDR_LEVEL; }
	const char *GetMenuName() const { return "Level"; }
	bool Render(RECT position);

private:
	bool LoadLevel(unsigned id);
	bool LoadSprites();
	bool DrawSprites(HDC hdc, RECT position, int base_x, int base_y);

	LEVELHEADER		_level_header;
	LEVELTEMPLATE	_level_template;
	unsigned		_level_id;
	Registers		_level_regs;

	unsigned		_level_sprites_total;
	SPRITES			_level_sprites;
	VRAM			_level_vram;
	Sprite			_level_sprites_gfx[500];
};

#endif
