
#include <windows.h>
#include "registers.h"
#include "rom.h"
#include "vram.h"

void Registers::SetRegVal(unsigned reg, unsigned val)
{
	if (FIRST_REGISTER > reg
		|| FIRST_REGISTER + TOTAL_REGISTER <= reg)
		return ; //out of rang
	unsigned index = reg - FIRST_REGISTER;
	regs[index] = val;
}

unsigned Registers::GetRegVal(unsigned reg)
{
	if (FIRST_REGISTER > reg
		|| FIRST_REGISTER + TOTAL_REGISTER < reg)
		return 0;
	return regs[reg - FIRST_REGISTER];
}

unsigned GetTileMapSizeByMode(unsigned mode)
{
	unsigned size = 1024;
	if (mode & 0x01)
		size *= 2;
	if (mode & 0x02)
		size *= 2;
	return size;
}

VRAMADDR Registers::GetTileMapVAddr(unsigned bg)
{
	unsigned val = GetRegVal(0x2107 + bg);
	unsigned vAddr = (val & 0xFC) << 9;
	unsigned vSize = GetTileMapSizeByMode(val & 0x03);
	return (val & 0xFC) << 9;
}

VRAMADDR Registers::GetTileGfxVAddr(unsigned bg)
{
	unsigned off = bg / 2;
	unsigned bit = bg % 2;
	unsigned val = GetRegVal(0x210B + off);
	val = (val >> (bit * 4)) & 0x0F;
	return val << 13;
}

bool Registers::Load(unsigned id)
{
	ROMFILE rf = rom.GetROMFILE();
	rf.FollowPtr2(0x3D79E2, id * 2);
	for (int i = 0; i < TOTAL_REGISTER; i++)
		regs[i] = 0x00;
	unsigned reg = 1;
	if (!rf.Read(&reg, 2))
		return false;
	do
	{
		if (reg & 0x8000)
		{
			//$BB/8C9C
			unsigned byte = (reg & 0xFF00) >> 8;
			byte &= 0x7F;
			byte <<= 8;
			reg &= 0x00FF;
			reg |= byte;
			unsigned values = 0;
			if (!rf.Read(&values, 2))
				return false;
			unsigned value1 = (values & 0x00FF);
			unsigned value2 = (values & 0xFF00) >> 8;
			SetRegVal(reg, value1);
			SetRegVal(reg + 1, value2);
		}
		else
		{
			//$BB/8CA9
			unsigned value = 0;
			if (!rf.Read(&value, 1))
				return false;
			SetRegVal(reg, value);
		}
		if (!rf.Read(&reg, 2))
			return false;
	} while (reg);
	return true;
};
