
#include <windows.h>
#include "rom.h"

ROM rom;

/*
 * No location speficied, query the user to get the location of the file then open the file.
 */
bool ROM::OpenFile(HWND hwnd)
{
	char location[MAX_PATH] = { 0 };
	OPENFILENAME of;
	of.lStructSize = sizeof(of);
	of.hwndOwner = hwnd;
	of.hInstance = (HINSTANCE) GetModuleHandle(NULL);
	of.lpstrFilter = "Super Nintendo Entertainment System Image\0*.smc\0All Files\0*.*\0";
	of.lpstrCustomFilter = NULL;
	of.nMaxCustFilter = 0;
	of.nFilterIndex = 1;
	of.lpstrFile = location;
	of.nMaxFile = sizeof(location);
	of.nFileExtension = 0;
	of.lpstrFileTitle = NULL;
	of.nMaxFileTitle = 0;
	of.lpstrInitialDir = NULL;
	of.lpstrTitle = NULL;
	of.Flags = /*OFN_ENABLEHOOK | */OFN_ENABLESIZING | OFN_HIDEREADONLY | OFN_EXPLORER/* | OFN_ENABLETEMPLATE*/;
	of.nFileOffset = 0;
	of.lpstrDefExt = ".smc";
	of.lpfnHook = NULL;
	of.lpTemplateName = NULL;
	of.lCustData = NULL;
	if (GetOpenFileName(&of)) {
		return OpenFile(location);
	}
	return false;
}
/*
 * Open the file to obtain an handle using the windows api.
 */
bool ROM::OpenFile(const char *location)
{
	HANDLE hFile = CreateFile(location, GENERIC_READ, NULL, NULL, OPEN_ALWAYS,
		NULL, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		return false;
	}
	_file.hFile = hFile;
	return true;
}

void ROM::CloseFile()
{
	if (_file.hFile != INVALID_HANDLE_VALUE) {
		CloseHandle(_file.hFile);
		_file.hFile = INVALID_HANDLE_VALUE;
	}
}

bool ROMFILE::UpdateCursor()
{
	return SetFilePointer(hFile, cAddr, NULL, FILE_BEGIN) != 0xFFFFFFFF;
}

bool ROMFILE::UpdateCursor(unsigned addr)
{
	cAddr = addr;
	return UpdateCursor();
}

bool ROMFILE::Read(void *data, unsigned len)
{
	if (!UpdateCursor())
		return false;
	DWORD read;
	if (ReadFile(hFile, data, len, &read, NULL) && read == len) {
		cAddr += len;
		return true;
	}
	return false;
}

bool ROMFILE::Read(unsigned addr, void *data, unsigned len)
{
	if (!UpdateCursor(addr))
		return false;
	return Read(data, len);
}

bool ROMFILE::FollowPtr2(unsigned offset)
{
	return Read(cAddr + offset, &cAddr, 2);
}

bool ROMFILE::FollowPtr2(unsigned addr, unsigned offset)
{
	unsigned newAddr = 0;
	bool retval = Read(addr + offset, &newAddr, 2);
	cAddr = newAddr + addr;
	return retval;
}
