
#ifndef _ROM_H_
#define _ROM_H_

#define FILE_MODE_READ		0x00
#define FILE_MODE_WRITE		0x01

/* Local's bank addr */
typedef unsigned short LOCALADDR;
typedef unsigned short VRAMADDR;

/*
 * HFILE contain an handle plus an adress of the current cursor position.
 */

struct ROMFILE
{
	ROMFILE() : hFile(INVALID_HANDLE_VALUE) { }
	// Read operation
	bool Read(void *data, unsigned len);
	bool Read(unsigned addr, void *data, unsigned len); // Change the cursor position to addr then do a read operation.
	// Write operation
	bool Write(void *data, unsigned len);
	bool Write(unsigned addr, void *data, unsigned len); // Change the cursor position then do a write operation
	bool UpdateCursor();
	bool UpdateCursor(unsigned addr);

	// Change the file mode to FILE_MODE_READ or FILE_MODE_WRITE.
	void ChangeFileMode(unsigned mode);
	bool RW(void *data, unsigned len);	// Write or read operation depends of the current file mode.
	bool RW(unsigned addr, void *data, unsigned len);

	// Read 2 bytes at addr + offset, append the current addr's bank, set the addr to.
	bool FollowPtr2(unsigned offset);
	bool FollowPtr2(unsigned addr, unsigned offset);
	// Read 3 bytes at addr + offset, set the addr to the read value.
	bool FollowPtr3(unsigned offset);
	bool FollowPtr3(unsigned addr, unsigned offset);

	unsigned cAddr;
	unsigned fileMode;
	HANDLE hFile;
};

/*
 * ROM manage the handle of the HFILE
 */

class ROM
{
public:
	ROM() { }
	~ROM() { }

	/* Query the user with the common dialog file picker to obtain a location of the file
	   hwnd is the window parent's handle */
	bool	OpenFile(HWND hwnd);
	/* Open a file to obtain a file handle. */
	bool	OpenFile(const char *location);
	void	CloseFile();

	ROMFILE	GetROMFILE() const { return _file; }

private:
	ROMFILE	_file;
};

// Global

extern ROM rom;

#endif /* _ROM_H_ */
