
#include <windows.h>
#include "sprites.h"
#include "rom.h"
#include "tile.h"

unsigned ReadTree(ROMFILE& rf) {
	unsigned i = 0;
	rf.Read(&i, 2);
	return i;
}

bool Sprite::Load(unsigned spriteid, int x, int y)
{
	if (!Load(spriteid))
		return false;

}

bool Sprite::Load(unsigned spriteid)
{
	ROMFILE rf = rom.GetROMFILE();
	unsigned sprInitAddr = 0x003F0000;
	if (!rf.Read(0x3BE800 + spriteid, &sprInitAddr, 2))
		return false;
	unsigned treeAddr[12] = {0};
	unsigned treeLvl = 0, treeCurObj = 0, animationid = 0, palid = 0x14;
	_animation_used = 0;
	treeAddr[0] = sprInitAddr;
	do {
		rf.Read(treeAddr[treeLvl], &treeCurObj, 2);
		treeAddr[treeLvl] += 2;
		if (treeCurObj & 0x8000) {
			treeCurObj >>= 8;
			switch (treeCurObj) {
			case 0x83: //0x06
				treeLvl++;
				treeAddr[treeLvl] = 0x003F0000;
				rf.Read(treeAddr[treeLvl - 1], treeAddr + treeLvl, 2);
				treeAddr[treeLvl - 1] += 2;
				break ;
			case 0x80:
				if (treeLvl == 0) {
					return true;
				}
				treeLvl--;
				break ;
			case 0x8D: // Load palette BB8678
				palid = 0;
				rf.Read(treeAddr[treeLvl], &palid, 2);
				treeAddr[treeLvl] += 2;
				break ;
			case 0x81: // Load an animation
				if (_animation_used < 5) {
					rf.Read(treeAddr[treeLvl], &animationid, 2);
					treeAddr[treeLvl] += 2;
					_animation[_animation_used++].Load(animationid, palid);
				}
				break ;
			case 0x8A:
				treeAddr[treeLvl] += 26;
				break ;
			case 0x85:
				treeAddr[treeLvl] += 2;
				break ;
			default:
				return false;
			}
		}
		else {
			unsigned t = 0;
			rf.Read(treeAddr[treeLvl], &t, 2);
			treeAddr[treeLvl] += 2;
			switch (treeCurObj) {
			case 0x1A:
				_animation[_animation_used++].LoadGFX(t, palid);
				break ;
			}
		}
	} while (1);
}

bool Sprite::Draw(HDC hdc, int x, int y)
{
	for (unsigned i = 0; i < _animation_used; i++) {
		if (!_animation[i].RenderNextFrame(hdc, x, y))
			return false;
	}
	return true;
}
