
#include <windows.h>
#include "tile.h"
#include "rom.h"
#include "sprites.h"

#define RESET_RECT(rect) rect.left = 0; rect.top = 0; rect.right = 0; rect.bottom = 0;

bool TileSubPalette::LoadSpriteSubPal(unsigned id) {
	unsigned palAddr = 0x003D0000;
	ROMFILE rfcd = rom.GetROMFILE();
	rfcd.Read(0x003D5FEE + id * 2, &palAddr, 2);
	return rfcd.Read(palAddr, color + 1, 30);
}

bool DrawTile8x8(HDC hdc, int x, int y, unsigned char tileData[32], TileSubPalette *subPal,
				 bool flipHoriz, bool flipVert, unsigned transColorIndex)
{
	for (unsigned i = 0; i < 8; i++)
	{
		unsigned char bp4 = tileData[i * 2 + 0x00]; // first byte of first row
		unsigned char bp3 = tileData[i * 2 + 0x01]; // second byte of first row
		unsigned char bp2 = tileData[i * 2 + 0x10]; // first byte of second row
		unsigned char bp1 = tileData[i * 2 + 0x11]; // second byte of second row
		for (unsigned n = 0; n < 8; n++)
		{

			unsigned color_index = ((bp4 & 0x80) >> 7) | ((bp3 & 0x80) >> 6)
				| ((bp2 & 0x80) >> 5) | ((bp1 & 0x80) >> 4);
			if (color_index != transColorIndex) {
				// Color is not transparent, we draw
				COLORREF color = subPal->GetColor(color_index)->GetRGB();
				unsigned x_pos = (flipHoriz) ? 7 - n : n;
				unsigned y_pos = (flipVert) ? 7 - i : i;
				SetPixel(hdc, x + x_pos, y + y_pos, color);
			}
			bp1 <<= 1;
			bp2 <<= 1;
			bp3 <<= 1;
			bp4 <<= 1;
		}
	}
	return true;
}

bool DrawTile8x8(HDC destDCTile, int x, int y, unsigned char *tileData, TileSubPalette *subPal,
				 bool flipHorizontal, bool flipVertical)
{
	unsigned char tileBits[192] = { 0x00 };
	unsigned bp1, bp2, bp3, bp4, i, x_pos, y_pos;
	HBITMAP bm8x8Tile;
	HDC		dc8x8Tile;
	bp1 = bp2 = bp3 = bp4 = 0;
	HDC cdc = GetDC(NULL);
	dc8x8Tile = CreateCompatibleDC(cdc);
	bm8x8Tile = CreateCompatibleBitmap(cdc, 8, 8);
	SelectObject(dc8x8Tile, (HGDIOBJ) bm8x8Tile);
	ReleaseDC(NULL, cdc);
	for (i = 0; i < 8; i++) {
		bp1 = tileData[i * 2 + 0x00], bp2 = tileData[i * 2 + 0x01];
		bp3 = tileData[i * 2 + 0x10], bp4 = tileData[i * 2 + 0x11];
		for (int c = 0, ex = 1;
			 c < 8;
			 c++, ex *= 2) {
				unsigned color_index = ((bp1 & 0x80) >> 7) | ((bp2 & 0x80) >> 6)
				| ((bp3 & 0x80) >> 5) | ((bp4 & 0x80) >> 4);
				x_pos = (flipHorizontal) ? 7 - c : c;
				y_pos = (flipVertical) ? 7 - i : i;
				COLORREF color = subPal->GetColor(color_index)->GetRGB();
				SetPixel(dc8x8Tile, x_pos, y_pos, color);
				bp1 <<= 1, bp2 <<= 1, bp3 <<= 1, bp4 <<= 1;
		}
	}
	BITMAPINFO bmi = { 0 };
	bmi.bmiHeader.biSize = sizeof(bmi);
	bmi.bmiHeader.biWidth = 8;
	bmi.bmiHeader.biHeight = -8;
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 24;
	bmi.bmiHeader.biClrUsed = 0;
	//BOOL c = SetDIBits(dc8x8Tile, bm8x8Tile, 0, 8, tileBits, &bmi, DIB_RGB_COLORS);
	BitBlt(destDCTile, x, y, 8, 8, dc8x8Tile, 0, 0, SRCCOPY);
	DeleteDC(dc8x8Tile);
	DeleteObject((HGDIOBJ) bm8x8Tile);
	return true;
}

bool DrawTileMap(HDC hdc, int x, int y, unsigned char *tileMap, unsigned char *tileData, TilePalette *pal)
{
	HBITMAP bmMask = CreateBitmap(8, 8, 1, 1, NULL);
	if (bmMask == NULL) {
		return false;
	}

	for (int i = 0; i < 1024; i++)
	{
		unsigned pos = i * 2;
		unsigned x_pos = (i % 32) * 8;
		unsigned y_pos = ((i / 32) % 32) * 8;
		unsigned gfxIndex = tileMap[pos] | ((tileMap[pos + 1] & 0x03) << 8 );
		unsigned subPaletteIndex = (tileMap[pos + 1] & 0x1C) >> 2;
		bool flipHorizontal = tileMap[pos + 1] & 0x40 ? true : false;
		bool flipVertical = tileMap[pos + 1] & 0x80 ? true : false;
		if (subPaletteIndex > 15) {
			return false;
		}
		TileSubPalette *subPal = (TileSubPalette*) (pal->color + subPaletteIndex * 16);
		DrawTile8x8(hdc, x + x_pos, y + y_pos, tileData + 32 * gfxIndex, subPal, flipHorizontal, flipVertical); 
	}
	return true;
}

bool DrawSprite(HDC hdc, unsigned gfxIndex, int x, int y, unsigned subPalId)
{
	ROMFILE rf = rom.GetROMFILE();
	unsigned gfxAddr = 0;
	if (!rf.Read(0x3C8000 + gfxIndex, &gfxAddr, 3))
		return false;
	gfxAddr -= 0xC00000;
	SPRITEGFXINFO si;
	if (!rf.Read(gfxAddr, &si, sizeof(si)))
		return false;
	POSITION pos16x16[20];
	POSITION pos8x8[20];
	int i;
	if (si.n16x16 > sizeof(pos16x16) / sizeof(POSITION))
		return false;
	if (si.n8x8 > sizeof(pos8x8) / sizeof(POSITION))
		return false;
	if (!rf.Read(pos16x16, si.n16x16 * sizeof(POSITION)))
		return false;
	if (!rf.Read(pos8x8, si.n8x8 * sizeof(POSITION)))
		return false;
	unsigned char data[4096];
	unsigned char *dataPos = data;
	unsigned gfxTotalSize = si.n16x16 * 128 + si.n8x8 * 32;
	if (gfxTotalSize > sizeof(data))
		return false;
	if (!rf.Read(data, gfxTotalSize))
		return false;
	/*if (!LoadData(firstPackAddr, firstPackSize, 0x00))
		return false;
	if (!LoadData(secondPackAddr, secondPackSize, firstPackSize))
		return false;*/
	TileSubPalette subPal;
	subPal.LoadSpriteSubPal(subPalId);
	// Draw a 16x16 sprite to the destination.
	for (i = 0; i < si.n16x16; i++) {
		unsigned local_x = pos16x16[i].x + x;
		unsigned local_y = pos16x16[i].y + y;
		if (!DrawTile8x8(hdc, local_x + 0, local_y + 0, dataPos, &subPal, false, false, 0))
			return false;
		if (!DrawTile8x8(hdc, local_x + 8, local_y + 0, dataPos + 32, &subPal, false, false, 0))
			return false;
		dataPos += 64;
	}
	dataPos = data + si.start8x8 * 32;
	for (i = 0; i < si.n8x8; i++) {
		unsigned local_x = pos8x8[i].x + x;
		unsigned local_y = pos8x8[i].y + y;
		if (!DrawTile8x8(hdc, local_x, local_y, dataPos, &subPal, false, false, 0))
			return false;
		dataPos += 32;
	}
	for (i = 0; i < si.n16x16; i++) {
		unsigned local_x = pos16x16[i].x + x;
		unsigned local_y = pos16x16[i].y + y;
		if (!DrawTile8x8(hdc, local_x + 0, local_y + 8, dataPos, &subPal, false, false, 0))
			return false;
		if (!DrawTile8x8(hdc, local_x + 8, local_y + 8, dataPos + 32, &subPal, false, false, 0))
			return false;
		dataPos += 64;
	}
	return true;
}

bool Tile::Load(unsigned gfxIndex, unsigned palID)
{
	SPRITEGFXINFO si;
	unsigned gfxAddr = 0, gfxTotalSize, i;
	POSITION pos16x16[20], pos8x8[20];
	POSITION pos8x8b[10];
	RECT position;
	unsigned char data[4096], *dataPos = data;
	TileSubPalette subPal;

	ROMFILE rf = rom.GetROMFILE();
	/* Load the address of the gfx by index, including the bank */
	if (!rf.Read(0x3C8000 + gfxIndex, &gfxAddr, 3)) {
		return false;
	}
	/* Substract to get a file address */
	gfxAddr -= 0xC00000;
	/* Read the sprite header */
	if (!rf.Read(gfxAddr, &si, sizeof(si))) {
		return false;
	}
	/* Check if it has more tiles than our fixed limit */
	if (si.n16x16 > sizeof(pos16x16) / sizeof(POSITION)
		|| si.n8x8 > sizeof(pos8x8) / sizeof(POSITION)) {
		return false;
	}
	if (!rf.Read(pos16x16, si.n16x16 * sizeof(POSITION)))
		return false;
	if (!rf.Read(pos8x8, si.n8x8 * sizeof(POSITION)))
		return false;
	if (!rf.Read(pos8x8b, si.n8x8b * sizeof(POSITION)))
		return false;
 	/* Calculate the total size of the gfx data */
	gfxTotalSize = si.n16x16 * 128 + si.n8x8 * 32;
	/* Check if it is more than our buffer fixed limit */
	if (gfxTotalSize > sizeof(data)) {
		return false;
	}
	/* Read the graphic data */
	if (!rf.Read(data, gfxTotalSize)) {
		return false;
	}
	/* Load the sub palette */
	if (!subPal.LoadSpriteSubPal(palID)) {
		return false;
	}
	if (si.n16x16) {
		position.left = position.right = pos16x16[0].x;
		position.top = position.bottom = pos16x16[0].y;
		position.right += 16;
		position.bottom += 16;
	}
	else {
		position.left = position.right = pos8x8[0].x;
		position.top = position.bottom = pos8x8[0].y;
		position.right += 8;
		position.bottom += 8;
	}
	_transColor = subPal.GetColor(0)->GetRGB();
	/* Get the lowest and highest positions */
	for (i = 0; i < si.n16x16; i++) { // 16 x 16 tiles
		RECT size = { pos16x16[i].x, pos16x16[i].y, pos16x16[i].x + 16, pos16x16[i].y + 16 };
		if (size.left < position.left)
			position.left = size.left;
		if (size.right > position.right)
			position.right = size.right;
		if (size.top < position.top)
			position.top = size.top;
		if (size.bottom > position.bottom)
			position.bottom = size.bottom;
	}
	for (i = 0; i < si.n8x8; i++) {
		RECT size = { pos8x8[i].x, pos8x8[i].y, pos8x8[i].x + 8, pos8x8[i].y + 8 };
		if (size.left < position.left)
			position.left = size.left;
		if (size.right > position.right)
			position.right = size.right;
		if (size.top < position.top)
			position.top = size.top;
		if (size.bottom > position.bottom)
			position.bottom = size.bottom;
	}
	_width = abs(position.right - position.left);
	_height = abs(position.top - position.bottom);
	_left = min(position.left, position.right);
	_top = min(position.top, position.bottom);
	HDC cdc = GetDC(NULL);
	_dcTile = CreateCompatibleDC(cdc);
	if (_dcTile == NULL) {
		ReleaseDC(NULL, cdc);
		return false;
	}
	_bmTile = CreateCompatibleBitmap(cdc, _width, _height);
	ReleaseDC(NULL, cdc);
	if (_bmTile == NULL) {
		DeleteDC(_dcTile);
		return false;
	}
	SelectObject(_dcTile, (HGDIOBJ) _bmTile);
	RECT toFill = { 0, 0, _width, _height };
	SetBkColor(_dcTile, _transColor);
	ExtTextOut(_dcTile, 0, 0, ETO_OPAQUE, &toFill, NULL, 0, NULL);
	for (i = 0; i < si.n16x16; i++) {
		unsigned local_x = pos16x16[i].x - _left;
		unsigned local_y = pos16x16[i].y - _top;
		if (!DrawTile8x8(_dcTile, local_x, local_y, dataPos, &subPal, false, false))
			return false;
		if (!DrawTile8x8(_dcTile, local_x + 8, local_y, dataPos + 32, &subPal, false, false))
			return false;
		dataPos += 64;
	}
	dataPos = data + si.start8x8 * 32;
	for (i = 0; i < si.n8x8; i++) {
		unsigned local_x = pos8x8[i].x - _left;
		unsigned local_y = pos8x8[i].y - _top;
		if (!DrawTile8x8(_dcTile, local_x, local_y, dataPos, &subPal, false, false))
			return false;
		dataPos += 32;
	}
	for (i = 0; i < si.n16x16; i++) {
		unsigned local_x = pos16x16[i].x - _left;
		unsigned local_y = pos16x16[i].y - _top;
		if (!DrawTile8x8(_dcTile, local_x, local_y + 8, dataPos, &subPal, false, false))
			return false;
		if (!DrawTile8x8(_dcTile, local_x + 8, local_y + 8, dataPos + 32, &subPal, false, false))
			return false;
		dataPos += 64;
	}
	return true;
}

bool Tile::Draw(HDC destDC, int x, int y)
{
	HDC dcMask = CreateCompatibleDC(NULL);
	HBITMAP bmMask = CreateBitmap(_width, _height, 1, 1, NULL);
	SelectObject(dcMask, (HGDIOBJ) bmMask);
	SetBkColor(_dcTile, _transColor);
	BitBlt(dcMask, 0, 0, _width, _height, _dcTile, 0, 0, SRCCOPY);
	BitBlt(_dcTile, 0, 0, _width, _height, dcMask, 0, 0, SRCINVERT);
	SetTextColor(destDC, RGB(0, 0, 0));
	SetBkColor(destDC, RGB(255, 255, 255));
	BitBlt(destDC, x + _left, y + _top, _width, _height, dcMask, 0, 0, SRCAND);
	BitBlt(destDC, x + _left, y + _top, _width, _height, _dcTile, 0, 0, SRCPAINT);
	DeleteObject((HGDIOBJ) bmMask);
	DeleteDC(dcMask);
	return true;
}

bool Animation::Load(unsigned animationid, unsigned palid)
{
	ROMFILE rf = rom.GetROMFILE();
	_frame_total = 0, _frame_current = 0;
	unsigned availableFrames = sizeof(_tiles) / sizeof(_tiles[0]);
	unsigned interval = 0;
	LOCALADDR gfxAddr = 0;
	if (!rf.FollowPtr2(0x390000, animationid * 4))
		return false;
	while (interval != 0x80
		&& _frame_total < availableFrames)
	{
 		rf.Read(&interval, 1);
		if (!rf.Read(&gfxAddr, 2))
			return false;
		if (interval >= 0x80) {
			switch (interval) {
			case 0x8f:
				if (!rf.Read(&gfxAddr, 2))
					return false;
				break ;
			}
			continue ;
		}
		_tiles[_frame_total++].Load(gfxAddr, palid);
	}
	return true;
}

bool Animation::LoadGFX(unsigned gfxaddr, unsigned palid)
{
	unsigned availableFrames = sizeof(_tiles) / sizeof(_tiles[0]);
	if (_frame_total >= availableFrames)
		return false;
	return _tiles[_frame_total++].Load(gfxaddr, palid);
}

bool Animation::RenderNextFrame(HDC hdc, int x, int y)
{
	RECT last_zone = _tiles[_frame_current].GetPosition(x, y);
	SetBkColor(hdc, RGB(0, 0, 0));
	ExtTextOut(hdc, x, y, ETO_OPAQUE, &last_zone, NULL, 0, 0);
	_frame_current++;
	if (_frame_current >= _frame_total)
		_frame_current = 0;
	_tiles[_frame_current].Draw(hdc, x, y);
	return true;
}

void Tile::Delete()
{
	if (_dcTile) {
		DeleteDC(_dcTile);
		_dcTile = NULL;
	}
	if (_bmTile) {
		DeleteObject((HGDIOBJ) _bmTile);
		_bmTile = NULL;
	}
}
