
#ifndef _TILE_H_
#define _TILE_H_

#pragma pack(push, 1)

struct TileColor
{
	/* Convert from 5 bits color to 8 bits */
	COLORREF GetRGB() { return RGB(
		GetRed() / 32.000 * 255,
		GetGreen() / 32.000 * 255,
		GetBlue() / 32.000 * 255
		); }
	BYTE GetRed() { return color[0] & 0x1F; }
	BYTE GetGreen() { return ((color[1] & 0x03) << 3) | (color[0] >> 5); }
	BYTE GetBlue() { return color[1] >> 2; }
	unsigned char color[2];
};

struct TileSubPalette
{
	TileColor	color[16];

	TileColor	*GetColor(unsigned index) { return color + index; }
	bool		LoadSpriteSubPal(unsigned id);
};

/* 128 colors palette (doesnt include sprites color data.) */
struct TilePalette
{
	TileColor	color[128]; // a color, 0bbbbbgg gggrrrrr x 128
	TileSubPalette *GetSubPalette(unsigned index) { return (TileSubPalette*) (color + index * 16); }
};

/* Used to draw a sprite */

struct POSITION
{
	unsigned char x;
	unsigned char y;
};

struct SPRITEGFXINFO
{
	unsigned char	n16x16;
	unsigned char	n8x8;
	unsigned char	start8x8;
	unsigned char	n8x8b;
	unsigned char	un03;
	unsigned char	firstPackSize;
	unsigned char	un04;
	unsigned char	un05;
};

#pragma pack(pop)

class Tile
{
public:
	Tile() : _bmTile(NULL), _dcTile(NULL) { }
	~Tile() { }
	bool		Load(unsigned gfxIndex, unsigned palid);
	bool		Draw(HDC destDC, int x, int y);
	void		Delete();
	unsigned	GetWidth() { return _width; }
	unsigned	GetHeight() { return _height; }
	RECT		GetPosition(int x, int y) {
		RECT position  = {x + _left, y + _top, x + _left + _width, y + _top + _height };
		return position;
	}

private:
	COLORREF	_transColor;
	HBITMAP		_bmTile;
	HDC			_dcTile;
	int			_top;
	int			_left;
	unsigned	_width;
	unsigned	_height;
};

class Animation
{
public:
	Animation() : _frame_total(0), _frame_current(0) { }
	~Animation() { }

	bool		Load(unsigned animationid, unsigned palid);
	bool		LoadGFX(unsigned gfxaddr, unsigned palid);
	bool		RenderFrame(unsigned frameid);
	bool		RenderNextFrame(HDC hdc, int x, int y);

private:
	unsigned	_frame_total;
	unsigned	_frame_current;
	unsigned	_palid;
	Tile		_tiles[20];
};

bool DrawTile8x8(HDC hdc, int x, int y, unsigned char tileData[32], TileSubPalette *subPal,
				 bool flipHorz, bool flipVert, unsigned transColorIndex);
bool DrawTile8x8(HDC hdc, int x, int y, unsigned char tileData[32], TileSubPalette *subPal,
				 bool flipHorz, bool flipVert);

bool DrawTileMap(HDC hdc, int x, int y, unsigned char *tileMap, unsigned char *tileData, TilePalette *pal);
bool DrawSprite(HDC hdc, unsigned spriteID, int x, int y);
bool DrawSprite(HDC hdc, unsigned gfxIndex, int x, int y, unsigned subPalId);

#endif
