
#include <windows.h>
#include "vram.h"
#include "rom.h"
#include "registers.h"
#include "tile.h"

bool VRAM::LoadPack(unsigned id)
{
	COLORREF test = _palette.color[0].GetBlue();
	ROMFILE rf = rom.GetROMFILE();
	rf.FollowPtr2(0x3D819A, id * 2);
	while (LoadPack(rf));
	return true;
}

bool VRAM::LoadPack(ROMFILE &rf)
{
	GFXPACK pack;
	rf.Read(&pack, sizeof(pack));
	if (pack.srcBank == 0)
		return false;
	unsigned vAddr = (pack.destAddr & 0x7fff) << 1;
	unsigned char *vPtr = _data + vAddr;
	unsigned vSize = pack.destSize;
	bool is_compressed = pack.destAddr & 0x8000 ? true : false;
	unsigned srcAddr = ((pack.srcBank - 0xC0) << 16) |  pack.srcAddr;
	if (is_compressed) {
		unsigned char *compressed_data = (unsigned char *) HeapAlloc(GetProcessHeap(), NULL, vSize);
		ROMFILE rfcd = rom.GetROMFILE();
		rfcd.Read(srcAddr, compressed_data, vSize);
		bool result = decompress(vPtr, vSize, compressed_data, vSize) != vSize;
		HeapFree(GetProcessHeap(), NULL, compressed_data);
		if (!result)
		{
			//return false;
		}
	}
	else
	{
		LoadData(srcAddr, vSize, vAddr);
	}
	return true;
}

bool VRAM::LoadData(unsigned srcAddr, unsigned vSize, VRAMADDR vAddr)
{
	ROMFILE rfcd = rom.GetROMFILE();
	return rfcd.Read(srcAddr, _data + vAddr, vSize);
}

bool VRAM::LoadPalette(unsigned srcAddr)
{
	ROMFILE rfcd = rom.GetROMFILE();
	return rfcd.Read(srcAddr, &_palette, sizeof(_palette));
}

bool VRAM::DrawBG2(HDC hdc, int x, int y, int width, int height)
{
	if (!_hBG2bm) {
		HDC cdc = GetDC(NULL);
		_hBG2bm = CreateCompatibleBitmap(cdc, 256, 256);
		if (_hBG2bm == 0) {
			ReleaseDC(NULL, cdc);
			return false;
		}
		_hBG2dc = CreateCompatibleDC(cdc);
		ReleaseDC(NULL, cdc);
		if (_hBG2dc == 0) {
			DeleteObject((HGDIOBJ) _hBG2bm);
			_hBG2bm = NULL;
			return false;
		}
		SelectObject(_hBG2dc, (HGDIOBJ) _hBG2bm);
		VRAMADDR BGGfxVAddr = _regs->GetTileGfxVAddr(1);
		VRAMADDR BGMapVAddr = _regs->GetTileMapVAddr(1);
		if (!DrawTileMap(_hBG2dc, 0, 0, _data + BGMapVAddr, _data + BGGfxVAddr, &_palette)) {
			DeleteDC(_hBG2dc);
			DeleteObject((HGDIOBJ) _hBG2bm);
			_hBG2bm = NULL;
			return false;
		}
	}
	HBRUSH hBG2b = CreatePatternBrush(_hBG2bm);
	HGDIOBJ oldObj = SelectObject(hdc, (HGDIOBJ) hBG2b);
	BOOL a = Rectangle(hdc, x, y, width, height);
	SelectObject(hdc, oldObj);
	//BitBlt(hdc, x, y, 256, 256, _hBG2dc, 0, 0, SRCCOPY);

}

bool VRAM::RenderBG(HDC hdc, unsigned nBG, int x, int y)
{
	VRAMADDR BGGfxVAddr = _regs->GetTileGfxVAddr(nBG);
	VRAMADDR BGMapVAddr = _regs->GetTileMapVAddr(nBG);
	DrawTileMap(hdc, x, y, _data + BGMapVAddr, _data + BGGfxVAddr, &_palette);
	//DrawTile8x8(hdc, x, y, _data + BGvAddr, (TileSubPalette*) &_palette);
	return true;
}

unsigned fillBytes(unsigned char *in, unsigned char byte, unsigned times, unsigned max)
{
	unsigned i = 0;
	for (; i < times && i < max; i++)
		in[i] = byte;
	return i;
}

unsigned decompress(unsigned char *dataOut, unsigned dataOutLen, const unsigned char *dataIn, unsigned dataInLen)
{
	unsigned char commondword1 = dataIn[0x01]; // $52
	unsigned char commonbyte2 = dataIn[0x02]; // $53
	unsigned char commonbyte3 = dataIn[0x03]; // $56
	unsigned char commonbyte4 = dataIn[0x04]; // $57
	unsigned char commonword1 = dataIn[0x05]; // $54
	unsigned char commonword2 = dataIn[0x06]; // $54

	unsigned indexOut = 0x00;
	unsigned indexIn = 0x27; // Y: $27 = 39
	unsigned char d = dataIn[indexIn++]; // $46
	unsigned char t = (d & 0xF0) >> 2;	// $4E
	while (1)
	{
		switch (t)
		{
			case 0x10: // repeat n + 3 times the common bytes
			{
				unsigned char times = (d & 0x0F) + 3;
				indexOut += fillBytes(dataOut + indexOut, commondword1, times, dataOutLen - indexOut);
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x08:
			{
				unsigned char byte = dataIn[indexIn++];
				dataOut[indexOut++] = (byte / 0x10) | ((d % 0x10) << 4);
				d = dataIn[indexIn++];
				dataOut[indexOut++] =  ((byte % 0x10) << 4) | (d / 0x10);
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x4F:
			{
				d = dataIn[indexIn++];
				unsigned char times = (d / 0x10) + 3;
				indexOut += fillBytes(dataOut + indexOut, commondword1, times, dataOutLen - indexOut);
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x3F:
			{ // miss a condition: BB8E78 - OK
				d = dataIn[indexIn++];
				unsigned char times = (d / 0x10);
				// si times = 0, on arrte
				if (times == 0)
				{
					return indexOut;
				}
				unsigned char n;
				for (unsigned i = 0; i < times; i++)
				{
					n = dataIn[indexIn++];
					dataOut[indexOut++] = ((d % 0x10) << 4) | (n / 0x10);
					d = n;
				}
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x43:
			{
				dataOut[indexOut++] = dataIn[indexIn++];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x0C:
			{
				unsigned char times = (d & 0x0F) + 3;
				d = dataIn[indexIn++];
				indexOut += fillBytes(dataOut + indexOut, d, times, dataOutLen - indexOut);
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x04:
			{
				unsigned char byte = dataIn[indexIn++];
				dataOut[indexOut++] = ((d % 0x10) << 4) | (byte / 0x10);
				d = byte;
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x73:
			{
				dataOut[indexOut++] = dataOut[indexOut - 1];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x4B:
			{
				d = dataIn[indexIn++];
				unsigned char times = d / 0x10 + 0x03;
				unsigned char byte = dataIn[indexIn++];
				indexOut += fillBytes(dataOut + indexOut,
					((d % 0x10) << 4) | (byte / 0x10), times, dataOutLen - indexOut);
				d = byte;
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x34:
			{
				dataOut[indexOut++] = dataOut[indexOut - 1];
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x57:
			{
				dataOut[indexOut++] = commonword1;
				dataOut[indexOut++] = commonword2;
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x28:
			{
				unsigned char times = (d & 0x0F) + 0x03;
				unsigned dataPos = indexOut - times - dataIn[indexIn++];
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[dataPos++];
				}
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x47:
			{
				dataOut[indexOut++] = dataIn[indexIn++];
				dataOut[indexOut++] = dataIn[indexIn++];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x24:
			{
				unsigned pos = (indexOut - ((d & 0x0F) + 0x02));
				dataOut[indexOut++] = dataOut[pos++];
				dataOut[indexOut++] = dataOut[pos];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x63:
			{
				d = dataIn[indexIn++];
				unsigned pos = indexOut - (d / 0x10 + 0x02);
				dataOut[indexOut++] = dataOut[pos++];
				dataOut[indexOut++] = dataOut[pos];
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x1C:
				dataOut[indexOut++] = commonbyte3;
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			case 0x3C:
			{
				unsigned pos = ((d & 0x0F) << 1) + 0x07;
				dataOut[indexOut++] = dataIn[pos++];
				dataOut[indexOut++] = dataIn[pos];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x20:
			{
				dataOut[indexOut++] = commonbyte4;
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x7B:
			{
				d = dataIn[indexIn++];
				unsigned pos = ((d & 0xF0) >> 3) + 0x07;
				dataOut[indexOut++] = dataIn[pos++];
				dataOut[indexOut++] = dataIn[pos];
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x5F:
			{
				dataOut[indexOut++] = commonbyte4;
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x00:
			{ //miss a condition
				unsigned times = d & 0x0F;
				if (!times)
				{
					return indexOut;
				}
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataIn[indexIn++];
				}
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x67:
			{
				unsigned char v = dataIn[indexIn++];
				unsigned times = (v / 16 + 0x03);
				d = dataIn[indexIn++];
				unsigned diff = times + (((v % 16) << 4) | (d / 16));
				unsigned pos = indexOut - diff;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[pos++];
				}
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x5B:
			{
				dataOut[indexOut++] = commonbyte3;
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x18:
			{
				dataOut[indexOut++] = commonword1;
				dataOut[indexOut++] = commonword2;
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x6B:
			{
				d = dataIn[indexIn++];
				unsigned times = ((d / 0x10) + 0x03);
				unsigned v = dataIn[indexIn++];
				unsigned diff = (((d & 0x0F) << 8) | v) + 0x0103;
				unsigned pos = indexOut - diff;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[pos++];
				}
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x2C:
			{
				unsigned times = (d & 0x0F) + 0x03;
				unsigned y = dataIn[indexIn++];
				d = dataIn[indexIn++];
				unsigned pos = indexOut - ((((y << 8) | d) >> 4) + 0x0103);
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[pos++];

				}
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x14:
			{
				unsigned times = (d & 0x0F) + 0x03;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = commonbyte2;
				}
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x53:
			{
				d = dataIn[indexIn++];
				unsigned times = (d / 0x10) + 0x03;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = commonbyte2;
				}
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x6F:
			{
				d = dataIn[indexIn++];
				unsigned times = (d / 0x10) + 0x03;
				unsigned y = dataIn[indexIn++];
				unsigned diff = (((d << 8) | y) << 4) & 0xffff;
				d = dataIn[indexIn++];
				diff |= (d / 0x10) & 0xff;
				unsigned pos = indexOut - diff;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[pos++];
				}
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x77:
			{
				dataOut[indexOut++] = dataOut[indexOut - 2];
				dataOut[indexOut++] = dataOut[indexOut - 2];
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			case 0x38:
			{
				dataOut[indexOut++] = dataOut[indexOut - 2];
				dataOut[indexOut++] = dataOut[indexOut - 2];
				t = ((d & 0x0F) << 2) + 0x3F;
				break ;
			}
			case 0x30:
			{
				unsigned times = (d & 0x0F) + 0x03;
				unsigned y = (dataIn[indexIn] << 8) | (dataIn[indexIn+1]);
				indexIn += 2;
				unsigned pos = indexOut - y;
				for (unsigned i = 0; i < times; i++)
				{
					dataOut[indexOut++] = dataOut[pos++];
				}
				d = dataIn[indexIn++];
				t = (d & 0xF0) >> 2;
				break ;
			}
			default:
				return indexOut;
		}
	}
	return indexOut;
}
