
#ifndef _VRAM_H_
#define _VRAM_H_

#include "rom.h"
#include "tile.h"

class Registers;

unsigned decompress(unsigned char *dataOut, unsigned dataOutLen, const unsigned char *dataIn, unsigned dataInLen);

#pragma pack(push, 1)

struct GFXPACK
{
	unsigned char	srcBank;
	LOCALADDR		srcAddr;
	VRAMADDR		destAddr;
	unsigned short	destSize;
};

#pragma pack(pop)

class VRAM
{
public:
	VRAM(Registers *regs) : _regs(regs), _hBG2bm(0), _hBG2dc(0) { }
	~VRAM() { }

	bool		LoadPack(unsigned id);
	bool		LoadData(unsigned srcAddr, unsigned vSize, VRAMADDR vAddr);
	bool		LoadPalette(unsigned srcAddr);
	// A sprite palette is 30 bytes size and is stored on 8th subPalette
	bool		LoadSpritePalette(unsigned id);
	bool		DrawBG2(HDC hdc, int x, int y, int width, int height);
	bool		DrawLevelSprite(HDC hdc, unsigned spriteid, int x, int y);
	bool		RenderBG(HDC hdc, unsigned nBG, int x, int y);
private:
	bool		LoadPack(ROMFILE &rf);
	unsigned char	_data[0x10000];
	TilePalette		_palette;
	HBITMAP			_hBG2bm;
	HDC				_hBG2dc;
	Registers		*_regs;
};

#endif
